function [Neigh_index trans_acc] = create_ngbrhood(MISCLASS, bg, ind, dim)
%function [Neigh_index trans_acc] = create_ngbrhood(MISCLASS, bg, ind, dim)
%[m n o] = dim(:);
m = dim(1);
n = dim(2);
o = dim(3);


%creating neighborhoods: Create a list of *all* pairs of neighbors, but
%don't include neighbors in the trim list.
Neigh_index = zeros(m*n*o*3,2);

%This matrix will hold the translation function from original coordinates
%to foreground coordinates.
trans_bg = zeros(m*n*o,1);

%disp('Creating full neighbor index list');

cntr=1;
fg = 1;
ibg = 1;
for k=1:m
    for j=1:n
        for i=1:o
            this = i + o*(j-1) + o * n * (k-1);
            
            %Cheap way of skipping bg voxels
            if this == bg(ibg)
                ibg = ibg + 1;
                continue;
            end
            
            %populate trans_bg
            trans_bg(this) = fg;
            fg = fg + 1;

            %3 cases... If nbr's index is less than 'this' then there
            %should already be a translation for it.
            nbr = this - 1;
            if i > 1 && trans_bg(nbr) ~= 0
               Neigh_index(cntr,1) = this;                
               Neigh_index(cntr,2) = nbr;
               cntr = cntr+1;
            end
            
            nbr = this - o;
            if j > 1 && trans_bg(nbr) ~= 0
               Neigh_index(cntr,1) = this;                
               Neigh_index(cntr,2) = nbr;
               cntr = cntr+1;
            end
              
            nbr = this - n * o;
            if k > 1 && trans_bg(nbr) ~= 0
               Neigh_index(cntr,1) = this;
               Neigh_index(cntr,2) = nbr;
               cntr = cntr+1;
            end
        end
    end
end


%So at this point, I have neighbor indices which point to neighbors in the
%original voxel space, and I need to translate the pointers into the
%foreground hypotheses space.

%disp('Translating coordinates');
%disp(sprintf('number of foreground Hypotheses: %d',max(find(Neigh_index(:,1) ~= 0))));

%Trim out unused portions of the Neighbor matrix.
Neigh_index(Neigh_index(:,1) == 0,:) = [];
Neigh_index(:,1) = trans_bg(Neigh_index(:,1));
Neigh_index(:,2) = trans_bg(Neigh_index(:,2));


%Now we have to remove the voxels which failed the T-test, and neighbor
%relations which fail the similarity test.

%Number of foreground voxels.
fg = prod(dim) - numel(bg);

%First remove links to bad nodes. ind is a list of entries to remove.
tr = 1:fg;
tr(ind) = 0;
Neigh_index(:,1) = tr(Neigh_index(:,1));
Neigh_index(:,2) = tr(Neigh_index(:,2));
Neigh_index(Neigh_index(:,1) == 0 | Neigh_index(:,2) == 0,:) = [];

%disp('Removing high error hypotheses and calculating coordinate translation');'
load params ngbr_width min_ngbrs;
tr(tr > 0) = 1:size(find(tr));
Neigh_index(abs(MISCLASS(tr(Neigh_index(:,1)), 1) - MISCLASS(tr(Neigh_index(:,2)), 1)) > ngbr_width,:) = [];


%Enforce that each voxel must participate in at least 2 neighbor
%constraints. Of course, some will participate in 2 neighbor constraints
%with nodes to be removed, but I think that overall this is an improvement.
remain = false(fg,1);

counts = zeros(fg,1);
z = unique(Neigh_index(:,1:2));
s = Neigh_index(:,1:2);
s = sort(s(:));
c=1;
for i=1:numel(z)
    while c <= numel(s) && s(c) == z(i)
        counts(z(i)) = counts(z(i)) + 1;
        c = c + 1;
    end
end

remain(counts >= min_ngbrs) = 1;
remain(ind) = 0;

trans_acc = zeros(fg,1);
trans_acc(remain) = 1:size(find(remain));

%Because some voxels may have been removed in the previous step, we now
%remove any neighbor relations invovling them. 
Neigh_index(:,1) = trans_acc(Neigh_index(:,1));
Neigh_index(:,2) = trans_acc(Neigh_index(:,2));
Neigh_index((Neigh_index(:,1) == 0 | Neigh_index(:,2) == 0),:) = [];
