function [err sens spec conf] = do_all_nfold_xval(n, folds)
%function [err sens spec conf] = do_all_nfold_xval(n, folds)

if nargin == 1
    folds = n;
end

%Flag to turn on or off various features
load params USE_FDG GM_pool;


if USE_FDG > 0
    disp('Loading FDG');
    load Data_FDG.mat Data_FDG s truth
    s = randperm(149);
    save s.mat s;
    Data = Data_FDG(s,:);
    clear Data_FDG;
    
    load bg_FDG.mat bg_FDG dim %trans_bg_FDG
    bg = bg_FDG;
else
    disp('Loading GM');
    load Data_GM.mat Data_GM s truth
    s = randperm(183);
    save s.mat s;
    Data = Data_GM(s,:);
    clear Data_GM;
    
    load bg.mat bg_GM dim %trans_bg
    bg = bg_GM;
end
truth = double(truth(s));


set_size = size(Data,1);
test_size = floor(set_size/n);
train_size = set_size - test_size;
Test_Train = [ones(set_size - test_size,1); 2*ones(test_size,1)];

err_all = cell(n,1);
w_separated = cell(n,1);
reduced_set = cell(n,1);

conf = zeros(size(truth));

total_err = 0;
total_test = 0;
total_TP = 0;
total_TP_FP = 0;
total_TP_FN = 0;

for i=1:folds
    Test_Train = [Test_Train(test_size+1:end); Test_Train(1:test_size)];
    train_set = find(Test_Train == 1);
    test_set = find(Test_Train == 2);
    
    
    pos = find(truth(train_set) == -1);
    neg = find(truth(train_set) == 1);

    truth_train = truth(train_set);
    truth_test = truth(test_set);
    

    %************
    [jnk pGM] = ttest2(Data(train_set(pos),:), Data(train_set(neg),:), 10^-8, 'both', 'unequal');
    [sort_pvals] = sort(pGM);
    SELECTED_SET = pGM <= sort_pvals(GM_pool);
    [SOLUTION] = create_clustering(Data(:,SELECTED_SET), truth, Test_Train);
    
    MISCLASS = abs(SOLUTION(:,train_set) * truth_train) / (2*train_size);
    [Neigh_index trans_acc] = create_ngbrhood(MISCLASS, bg, find(pGM > sort_pvals(GM_pool)), dim);
    fg_GM = find(trans_acc);
    reduced_set{i} = trans_acc;

    %write_nii('pFDG', dim, {-log(pGM)}, {1:size(Data,2)});
    
    %Some voxels may need to be be eliminated for not having any neighbors.
    SOLUTION(trans_acc(SELECTED_SET) == 0,:) = [];

    %Because some features are discarded for being orphans, we have to
    %first translate back to the original coordinate space, then down to
    %the final voxel set when passing training set error information to the
    %classifier. (Used for weighting the objective function).
    M = zeros(size(Data,1),1);
    M(SELECTED_SET) = MISCLASS;


    disp('#################');
    disp(sprintf('Fold %d', i));
    disp(sprintf('There are %d non-discounted neighbor constraints, and %d voxels.', size(Neigh_index,1), size(SOLUTION,1)));
    
    %Call CPLEX with the combined WM and GM data
    SOLUTION = SOLUTION'; %Have to transpose SOLUTION here.
    [w err] = call_cplex(SOLUTION(train_set,:), truth_train, SOLUTION(test_set,:), truth_test, Neigh_index, M(fg_GM));
    err_all{i} = err';
    
    w_separated{i} = w;
    
    save latest.mat w_separated err_all reduced_set;
 
    
    %Calculate some error metrics
    if size(find(w),1) > 0
        pred = SOLUTION(test_set,:) * w;
        conf(Test_Train == 2) = sigmoid(SOLUTION(test_set,:) * w, 0.5);
        err_final = sign(pred) - double(truth_test);
        
        disp(sprintf('Error rate on fold %d using raw weights: %f', i, sum(abs(err_final))/(2*numel(truth_test))));
        total_err = total_err + sum(abs(err_final))/2;
        
        %Remember, -1 = alzheimer's.
        total_TP = total_TP + numel(find(pred < 0 & truth_test == -1));
        total_TP_FN = total_TP_FN + numel(find(truth_test == -1));
        total_TP_FP = total_TP_FP + numel(find(pred < 0));
        if total_TP > 0
            disp(sprintf('Total Sensitivity so far: %f', total_TP/total_TP_FN)); %TP/(TP + FN)
            disp(sprintf('Total Specificity so far: %f', total_TP/total_TP_FP)); %TP/(TP + FP)
        end

        total_test = total_test + numel(truth_test);
        disp(sprintf('Total error rate so far: %f', total_err / total_test));
    else
        disp('No weights computed.');
    end
        
    %pause(1);
end

if total_test == 0
    err = 1;
    sens = 0;
    spec = 0;
else
    err = total_err / total_test;
    sens = total_TP/total_TP_FN;
    spec = total_TP/total_TP_FP;
end

%Unshuffle the confidence outputs
un_s = sortrows([(1:numel(s))' s'], 2);
conf = conf(un_s(:,1));

end %function
